/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function () {

    var myapp = {

        MAIN_PAGE: 'main-page',
        MAIN_PAGE_PATTERNS_LIST: 'patterns-list',
        MAIN_PAGE_SETTINGS_BUTTON: 'settings-button',

        SETTINGS_PAGE: 'settings-page',
        SETTINGS_PAGE_RADIO_BUTTON: 'radio-button',

        ERROR_POPUP: 'error-popup',
        ERROR_POPUP_MESSAGE: 'popup-message',
        ERROR_POPUP_BUTTON: 'popup-button',

        patterns: {

            TAP: 'TAP',
            SIP: 'SIP',
            KEY0: 'KEY0',
            KEY1: 'KEY1',
            KEY2: 'KEY2',
            KEY3: 'KEY3',
            KEY4: 'KEY4',
            KEY5: 'KEY5',
            KEY6: 'KEY6',
            KEY7: 'KEY7',
            KEY8: 'KEY8',
            KEY9: 'KEY9',
            KEY_STAR: 'KEY_STAR',
            KEY_SHARP: 'KEY_SHARP',
            BACK: 'KEY_BACK',
            HOLD: 'HOLD',
            HW_TAP: 'HW_TAP',
            HW_HOLD: 'HW_HOLD',
            MESSAGE: 'MESSAGE',
            EMAIL: 'EMAIL',
            WAKEUP: 'WAKEUP',
            SCHEDULE: 'SCHEDULE',
            TIMER: 'TIMER',
            GENERAL: 'GENERAL',
            POWERON: 'POWERON',
            POWEROFF: 'POWEROFF',
            CHARGERCONN: 'CHARGERCONN',
            CHARGING_ERROR: 'CHARGING_ERROR',
            FULLCHARGED: 'FULLCHARGED',
            LOWBATT: 'LOWBATT',
            LOCK: 'LOCK',
            UNLOCK: 'UNLOCK',
            VIBRATION_ON: 'VIBRATION_ON',
            SILENT_OFF: 'SILENT_OFF',
            BT_CONNECTED: 'BT_CONNECTED',
            BT_DISCONNECTED: 'BT_DISCONNECTED',
            LIST_REORDER: 'LIST_REORDER',
            LIST_SLIDER: 'LIST_SLIDER',
            VOLUME_KEY: 'VOLUME_KEY'

        },

        data: {
            pattern: null,
            type: 'TYPE_SOUND', // default type is 'TYPE_SOUND'
            _listHelper: null
        },

        ui: {
            mainpage: {
                createList: {},
                patternslist: {},
                settingsbutton: {}
            },
            settingspage: {
                radiobutton: {}
            },
            errorpopup: {
                message: {},
                button: {}
            }
        },

        play: {}

    };

    var init = function () {

        var self = this;

        /**
         *  Returns an element for myapp.ui.mainpage.patternslist .
         *  @private
         */
        self.ui.mainpage.patternslist = (function (id) {
            return document.querySelector('#' + id);
        })(self.MAIN_PAGE_PATTERNS_LIST);

        /**
         *  Returns an element for myapp.ui.mainpage.settingsbutton .
         *  @private
         */
        self.ui.mainpage.settingsbutton = (function (id) {
            return document.querySelector('#' + id);
        })(self.MAIN_PAGE_SETTINGS_BUTTON);

        /**
         *  Returns an element for myapp.ui.mainpage.radiobutton .
         *  @private
         */
        self.ui.settingspage.radiobutton = (function (id) {
            return document.querySelector('#' + id);
        })(self.SETTINGS_PAGE_RADIO_BUTTON);

        /**
         *  Returns an element for myapp.ui.errorpopup.message .
         *  @private
         */
        self.ui.errorpopup.message = (function (id) {
            return document.querySelector('#' + id);
        })(self.ERROR_POPUP_MESSAGE);

        /**
         *  Returns an element for myapp.ui.errorpopup.button .
         *  @private
         */
        self.ui.errorpopup.button = (function (id) {
            return document.querySelector('#' + id);
        })(self.ERROR_POPUP_BUTTON);

        /**
         *  Append elements to the list of mainpage.
         *  @public
         */
        self.ui.mainpage.createList = function () {

            var attr = 0, li, a;
            for (attr in self.patterns) {

                li = document.createElement('li');
                a = document.createElement('a');
                a.id = self.patterns[attr].replace('_', '-').toLowerCase();
                a.innerHTML = self.patterns[attr];
                li.appendChild(a);

                self.ui.mainpage.patternslist.appendChild(li);

            }

        }

        /**
         *  Plays a feedback by myapp.data.pattern and myapp.data.mode .
         *  @public
         */
        self.play = function () {

            try {

                /**
                 *  Checks whether a selected pattern is supported.
                 *  If not supported, error-popup is opened.
                 */
                var isSupported = false;
                isSupported = tizen.feedback.isPatternSupported(self.data.pattern, self.data.type);
                if (isSupported === false) {
                    self.ui.errorpopup.message.innerHTML = 'Not Supported Pattern';
                    tau.openPopup('#' + self.ERROR_POPUP);
                    return;
                }

                /**
                 *  If supported, paly the pattern.
                 */
                tizen.feedback.play(self.data.pattern, self.data.type);

            } catch (error) {

                self.ui.errorpopup.message.innerHTML = error.name + ': ' + error.message;
                tau.openPopup('#' + self.ERROR_POPUP);

            }

        };

    };


    window.onload = function () {

        init.call(myapp);

        window.addEventListener('tizenhwkey', function (event) {

            if (event.keyName === 'back') {

                var page = document.getElementsByClassName('ui-page-active')[0],
                    pageid = page ? page.id : '';
                if (pageid === 'main-page') {

                    tizen.application.getCurrentApplication().exit();

                } else {

                    window.history.back();

                }

            }

        });

        /**
         *  To handle rotary events
         */
        if (tau.support.shape.circle) {

            document.addEventListener('pagebeforeshow', function (event) {

                var id = event.target.id;

                if (id === myapp.MAIN_PAGE) {

                    myapp.data._listHelper = tau.helper.SnapListStyle.create(myapp.ui.mainpage.patternslist);

                } else if (id === myapp.SETTINGS_PAGE) {

                    myapp.data._listHelper = tau.helper.SnapListStyle.create(myapp.ui.settingspage.radiobutton);

                }

            });

            document.addEventListener('pagebeforehide', function (event) {

                var id = event.target.id;

                if (myapp.data._listHelper !== null) {

                    myapp.data._listHelper.destroy();

                }

            });

        }

        /**
         *  This handler function will be called by selecting a pattern on the list of main-page.
         */
        myapp.ui.mainpage.patternslist.onclick = function (event) {

            var id = event.target.id;

            /**
             *  If the event target id is 'patterns-list' it will be ignored.
             */
            if (id === myapp.MAIN_PAGE_PATTERNS_LIST) {

                return;

            }

            myapp.data.pattern = document.querySelector('#' + id).innerHTML;

            myapp.play();

        };

        /**
         *  This handler function will be called by clicking the Settins button on main-page.
         */
        myapp.ui.mainpage.settingsbutton.onclick = function () {

            tau.changePage(myapp.SETTINGS_PAGE);

        };

        /**
         *  This handler function will be called by selecting one of the radio buttons on settings-page.
         */
        myapp.ui.settingspage.radiobutton.onclick = function (event) {

            myapp.data.type = event.target.value;

        };

        /**
         *  This handler function will be called by clicking the OK button on error-popup.
         */
        myapp.ui.errorpopup.button.onclick = function () {

            tau.closePopup();

        };

        /**
         *  Create the list to display on main-page.
         */
        myapp.ui.mainpage.createList();

        tau.changePage(myapp.MAIN_PAGE);

    };

})();